#!/usr/bin/env python
import argparse
import warnings

import pandas as pd
from plotnine.exceptions import PlotnineWarning

from nanoCEM.cem_utils import identify_file_path, build_out_path, run_samtools
from nanoCEM.plot import alignment_plot

warnings.filterwarnings("ignore", category=PlotnineWarning)


def count_mis(row, feature_matrix):
    for item in feature_matrix:
        row[item] = row[4].count(item)
    return row


def init_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument("--chrom", help="Gene or chromosome name(head of your fasta file)")
    parser.add_argument("--pos", type=int, help="site of your interest")
    parser.add_argument("--len", default=10, type=int, help="region around the position")
    parser.add_argument('-t',"--cpu", default=4, type=int, help="region around the position")
    parser.add_argument("--strand", default="+", help="Strand of your interest")
    parser.add_argument('--rna', action='store_true', help='Turn on the RNA mode')
    parser.add_argument('-r', "--ref", help="fasta file")
    parser.add_argument('-i', "--input_fastq",
                        help="input_bam_file")
    parser.add_argument('-c', "--control_fastq",
                        help="control_bam_file")
    # parser_tombo.add_argument('-b', "--bam", help="bam file to help index to speed up")
    parser.add_argument('-o', "--output", default="nanoCEM_result", help="output_file")

    parser.set_defaults(chrom="NR_103073.1", pos=2030, len=10, strand='+', input_fastq='../example/data/wt/file.fastq',
                        control_fastq='../example/data/ivt/file.fastq', output='tombo_result_rna', rna=True,cpu=16,
                        ref='../example/data/23S_rRNA.fasta')
    # parser.set_defaults(chrom="NR_103073.1", pos=875, len=10, strand='-', input_bam='../example/data/reverse/wt.bam',
    #                     control_bam='../example/data/reverse/ivt.bam', output='tombo_result_rna_re', rna=True,
    #                     ref='../example/data/reverse/23S_rRNA_re.fasta')
    return parser


if __name__ == '__main__':
    # Parse the arguments
    parser = init_parser()
    args = parser.parse_args()
    input_fastq = args.input_fastq
    control_fastq = args.control_fastq
    ref = args.ref
    cpu = str(args.cpu)
    # check input file
    identify_file_path(input_fastq)

    # prepare title
    location = args.chrom + ':' + str(args.pos - args.len) + '-' + str(args.pos + args.len)
    title = location + ':' + args.strand
    # check output path
    results_path = args.output
    build_out_path(results_path)

    print("Running samtools ...")
    sample_feature = run_samtools(input_fastq, location, ref, results_path, 'Sample',cpu)
    if sample_feature.shape[0]<args.len*2+1:
        raise Exception('Invalid position selection: exceeds reference length or contains positions with no coverage.')
    pos_list = list(sample_feature[1].values)
    base_list = list(sample_feature[2].values)
    try:
        identify_file_path(control_fastq)
        control_feature = run_samtools(control_fastq, location, ref, results_path, 'Control',cpu)
        final_feature = pd.concat([sample_feature, control_feature], axis=0)
    except Exception:
        print("control files do not exist, will turn to the single mode")
        final_feature = sample_feature
        final_feature['Group'] = 'Single'

    if args.rna:
        base_list = (''.join(base_list)).replace("T", "U")
        base_list = list(base_list)
        title = 'RNA  ' + title
    else:
        title = 'DNA  ' + title

    print("Calculating alignment feature...")

    if args.strand == '+':
        feature_matrix = ['.', 'A', 'T', 'C', 'G']
    else:
        feature_matrix = [",", 'a', 't', 'c', 'g']


    final_feature = final_feature.apply(lambda x: count_mis(x, feature_matrix), axis=1)
    final_feature['Match'] = final_feature[feature_matrix[0]]
    final_feature.drop(feature_matrix[0], inplace=True, axis=1)
    final_feature.columns = ['Chrom', 'Position', 'Base', 'Coverage', 'Align string', 'Q string', 'Group', 'A', 'T',
                             'C', 'G', 'Match']
    final_feature.to_csv(results_path + '/alignment_feature.csv', index=None)
    print("Alignment feature file saved as "+results_path + '/alignment_feature.csv')

    final_feature = pd.melt(final_feature, id_vars=['Position', 'Group'], value_vars=['A', 'T', 'C', 'G', 'Match'])

    print("Start to plot ...")
    alignment_plot(final_feature, pos_list, base_list, title, args.pos, results_path)
    print('Finished')




import numpy as np
from matplotlib import pyplot as plt
import pandas as pd
import xgboost
import shap
import torch
import torch.nn as nn
import torch.optim as optim


# 5 is time step 4 is the feature number
X = feature_matrix.values.reshape(-1,5,4)
y = label.values
X = torch.from_numpy(X)
y = torch.from_numpy(y).double()
print(1)


class RNNClassifier(nn.Module):
    def __init__(self, input_size, hidden_size, num_layers, output_size):
        super(RNNClassifier, self).__init__()
        self.hidden_size = hidden_size
        self.num_layers = num_layers

        # RNN层
        self.rnn = nn.RNN(input_size, hidden_size, num_layers, batch_first=True).double()
        # 全连接层
        self.fc = nn.Linear(hidden_size, output_size).double()

    def forward(self, x):
        out, _ = self.rnn(x)
        out = self.fc(out[:, -1, :])  # 只使用最后一个时间步的输出
        out = torch.sigmoid(out)
        return out



# 设置模型的输入、隐藏层大小、输出类别数
input_size = 4  # 输入特征的维度
hidden_size = 32  # 隐藏层大小
num_layers = 1  # RNN层数
output_size = 1

# 创建RNN分类器模型实例
model = RNNClassifier(input_size, hidden_size, num_layers, output_size)

# 准备数据和标签
# 定义损失函数和优化器
criterion = nn.BCELoss()
optimizer = optim.Adam(model.parameters(), lr=0.0001)

# 训练模型
num_epochs = 100
batch_size = 32

for epoch in range(num_epochs):
    for i in range(0, len(X), batch_size):
        inputs = X[i:i+batch_size]
        labels = y[i:i+batch_size]

        optimizer.zero_grad()
        outputs = model(inputs)
        loss = criterion(outputs, labels)
        loss.backward()
        optimizer.step()

    if (epoch+1) % 10 == 0:
        print(f'Epoch {epoch+1}/{num_epochs}, Loss: {loss.item()}')
model.eval()
# 使用训练好的模型进行预测
with torch.no_grad():
    inputs = X  # 使用全部数据进行预测
    predictions = model(inputs)
    predicted_labels = torch.round(predictions).squeeze().long()

    # 计算准确率
    correct = (predicted_labels == y).sum().item()
    total = len(y)
    accuracy = correct / total

    print("Accuracy: "+str(accuracy))

# 打印预测结果
print(predicted_labels)



from sklearn.metrics import accuracy_score
from sklearn.model_selection import KFold
kfold = KFold(n_splits=5)
# Perform k-fold cross-validation
accuracy_scores=[]
for train_index, test_index in kfold.split(X):
    # Split the data into training and testing sets for each fold
    X_train, X_test = X[train_index], X[test_index]
    y_train, y_test = y[train_index], y[test_index]

    # Create an XGBoost classifier
    model = xgboost.XGBClassifier()

    # Train the model on the training data
    model.fit(X_train, y_train)

    # Make predictions on the testing data
    y_pred = model.predict(X_test)

    # Calculate the accuracy score
    accuracy = accuracy_score(y_test, y_pred)

    # Store the accuracy score for this fold
    accuracy_scores.append(accuracy)

    # Calculate the average accuracy across all folds
avg_accuracy = sum(accuracy_scores) / len(accuracy_scores)
print(avg_accuracy)



y_pred = model.predict(X_test)
accuracy = accuracy_score(y_test, y_pred)
# explain the model's predictions using SHAP
# (same syntax works for LightGBM, CatBoost, scikit-learn, transformers, Spark, etc.)
print(accuracy)
y_pred = model.predict_proba(X_test)[:,1]
prediction_df = pd.DataFrame({'y_test': y_test.reshape(-1,), 'y_pred': y_pred})
prediction_df['y_test'] = prediction_df['y_test'].apply(lambda x: 'Sample' if x==1 else 'Control')
import plotnine as p9
category = pd.api.types.CategoricalDtype(categories=['Sample', "Control"], ordered=True)
prediction_df['y_test'] = prediction_df['y_test'].astype(category)
# visualize the first prediction's explanation
plot = p9.ggplot(prediction_df, p9.aes(x='y_test', y="y_pred",fill='y_test')) \
            +p9.scale_fill_manual(values={"Sample": "#F57070", "Control": "#9F9F9F", "Single": "#a3abbd"})\
           + p9.theme_bw() \
           + p9.labs(x='',y='Prediction')\
           + p9.geom_boxplot(width=0.6,alpha=0.7)\
           + p9.theme(
        figure_size=(4, 4),
        panel_grid_minor=p9.element_blank(),
        axis_text=p9.element_text(size=13),
        axis_title=p9.element_text(size=13),
        title=p9.element_text(size=13),
        legend_position='none',
        legend_title=p9.element_blank(),
        strip_text=p9.element_text(size=13),
        strip_background=p9.element_rect(alpha=0),
    )
print(plot)
plot.save(filename=results_path + "/prediction_barplot.pdf", dpi=300)
plot = p9.ggplot(prediction_df, p9.aes(fill='y_test', x="y_pred")) \
    +p9.scale_fill_manual(values={"Sample": "#F57070", "Control": "#9F9F9F", "Single": "#a3abbd"})\
           + p9.theme_bw() \
           + p9.labs(x='Prediction',y='Density')\
           + p9.geom_density(alpha=0.7)\
           + p9.theme(
        figure_size=(4,4),
        panel_grid_minor=p9.element_blank(),
        axis_text=p9.element_text(size=13),
        axis_title=p9.element_text(size=13),
        title=p9.element_text(size=13),
        legend_position='bottom',
        legend_title=p9.element_blank(),
        strip_text=p9.element_text(size=13),
        strip_background=p9.element_rect(alpha=0),
    )
print(plot)

plot.save(filename=results_path + "/prediction_distribution.pdf", dpi=300)


# 示例数据




"""
Signal to seQuence alignment Plot - plot
Hiruna Samarakoon - Garvan Medical Institute
hiruna@unsw.edu.au
"""
from bokeh.plotting import figure, show, output_file, save
from bokeh.models import BoxAnnotation, HoverTool, WheelZoomTool, ColumnDataSource, Label, LabelSet, Segment, Toggle, Range1d, FreehandDrawTool
import re
import numpy as np

PLOT_X_RANGE = 300
PLOT_HEIGHT = 600

#base_shift related
SIG_SAMPLES_LIMIT = 5000
BASE_INDEX = {'A': 0, 'C': 1, 'G': 2, 'T': 3, 'U': 3}

def adjust_before_plotting(ref_seq_len, signal_tuple, region_tuple, sig_algn_data, fasta_seq):
    if sig_algn_data["data_is_rna"]:
        ref_region_start_diff = region_tuple[1] - region_tuple[3]
    else:
        ref_region_start_diff = region_tuple[2] + 1 - region_tuple[0]
    ref_region_end_diff = region_tuple[2] + ref_seq_len - region_tuple[1]
    # print("ref_region_start_diff: " + str(ref_region_start_diff))
    moves = sig_algn_data['ss']
    if ref_region_start_diff < 0:
        # we have to remove part of the sequence until we get to the ref start
        count_bases = 0
        eat_signal = 0
        count_moves = 0
        prev_move = None
        updated_move = []
        for i in moves:
            if count_bases == abs(ref_region_start_diff):
                break
            if count_bases > abs(ref_region_start_diff):
                if not prev_move.find('D'):
                    raise Exception("Error: a deletion move was expected. incorrect implementation. Please report with a minimal reproducible test")
                updated_move = ["{}D".format(count_bases-abs(ref_region_start_diff))]
                break
            count_moves += 1

            if 'D' in i:
                i = re.sub('D', '', i)
                count_bases += int(i)
                # print(i+" D "+str(int(sam_record.reference_start) + 1 + count_bases))
            elif 'I' in i:
                i = re.sub('I', '', i)
                eat_signal += int(i)
                # print(i+" I "+str(int(sam_record.reference_start) + 1 + count_bases))
            else:
                eat_signal += int(i)
                count_bases += 1
            prev_move = i
        moves = updated_move + moves[count_moves:]
        x = signal_tuple[0][:-eat_signal]
        x_real = signal_tuple[1][eat_signal:]
        y = signal_tuple[2][eat_signal:]
        signal_tuple = (x, x_real, y)
        sig_algn_data['ss'] = moves
    return signal_tuple, region_tuple, sig_algn_data, fasta_seq
def create_figure(args, plot_mode):
    p_defualt = None
    if plot_mode == 0:
        y_axis_label = "signal value (raw)"
        if args.no_pa:
            y_axis_label = "signal value (pA)"
        tools_to_show = 'hover,box_zoom,pan,reset,save,wheel_zoom,zoom_in,zoom_out'
        p_default = figure(x_axis_label='signal index',
                   y_axis_label=y_axis_label,
                   sizing_mode="stretch_width",
                   height=PLOT_HEIGHT,
                   output_backend="webgl",
                   x_range=(0, PLOT_X_RANGE),
                   tools=tools_to_show,
                   toolbar_location="below")
        # tooltips=tool_tips)
        p_default.select(dict(type=WheelZoomTool)).maintain_focus = False
        p_default.toolbar.active_scroll = p_default.select_one(WheelZoomTool)
        p_default.toolbar.logo = None
    elif plot_mode == 1:
        tools_to_show = 'hover,box_zoom,pan,save,wheel_zoom,reset,zoom_in,zoom_out'
        p_default = figure(output_backend="webgl",
                    sizing_mode="stretch_both",
                   # sizing_mode="scale_width",
                   # sizing_mode="scale_height",
                   # height=PLOT_HEIGHT,
                   x_range=(0, PLOT_X_RANGE),
                   tools=tools_to_show)
        # tooltips=tool_tips)
        # p.yaxis.visible = False
        p_default.select(dict(type=WheelZoomTool)).maintain_focus = False
        p_default.toolbar.active_scroll = p_default.select_one(WheelZoomTool)
        p_default.toolbar.logo = None
    return p_default
def scale_signal(y, sig_scale):
    if sig_scale == "medmad":
        arr = np.ma.array(y).compressed()
        read_median = np.median(arr)
        if read_median == np.nan:
            raise Exception("Error: calculated median is NaN")
        mad = np.median(np.abs(arr - read_median))
        if mad == np.nan:
            raise Exception("Error: calculated mad is NaN")
        read_mad = mad * 1.4826
        if read_mad < 1.0:
            read_mad = 1.0
        y = (y - read_mad) / read_mad
    elif sig_scale == "znorm":
        # zsig = sklearn.preprocessing.scale(y, axis=0, with_mean=True, with_std=True, copy=True)
        # Calculate the z-score from scratch
        y = (y - np.mean(y)) / np.std(y)
    elif not sig_scale == "":
        raise Exception("Error: given --sig_scale method: {} is not supported".format(sig_scale))
    return y
def calculate_offset_values(moves, sequence, raw_signal, kmer_length, sig_move_offset):
    # print("sig_move_offset: {}".format(sig_move_offset))
    # print("kmer_length: {}".format(kmer_length))
    len_seq = len(sequence)
    test_array = []
    for offset in range(0, kmer_length):
        freq = [[], [], [], []]
        start_raw = 0
        for j in range(0, sig_move_offset):
            start_raw += int(moves[j])
        for i in range(0, len_seq-kmer_length + 1 - sig_move_offset):
            end_raw = start_raw + int(moves[i + sig_move_offset])
            value = raw_signal[start_raw: end_raw]
            start_raw = end_raw
            freq[BASE_INDEX[sequence[i + offset]]].append(np.median(value))
        test_array.append(freq)
    return test_array
def clean_signal(y, fasta_seq, moves):
    new_moves = []
    new_y = []
    new_fasta_seq = ""
    signal_index = 0
    base_index = 0
    for i in moves:
        if base_index >= len(fasta_seq):
            break
        if signal_index >= SIG_SAMPLES_LIMIT:
            break
        if 'D' in i:
            i = re.sub('D', '', i)
            count_bases = int(i)
            base_index += count_bases

        elif 'I' in i:
            i = re.sub('I', '', i)
            eat_signal = int(i)
            signal_index += eat_signal
        else:
            eat_signal = int(i)
            count_bases = 1
            new_moves.append(i)
            for j in y[signal_index:signal_index+eat_signal]:
                new_y.append(j)
            new_fasta_seq += fasta_seq[base_index]
            base_index += count_bases
            signal_index += eat_signal
    return new_y, new_fasta_seq, new_moves
def calculate_offset_distance(kmer_length, test_array):
    start_offset = 0
    end_offset = kmer_length
    # offset_dist = []
    # for offset in range(start_offset, end_offset):
    #     max_mean = -1
    #     min_mean = 10000
    #     for base in test_array[offset]:
    #         mean = np.median(base)
    #         if mean < min_mean:
    #             min_mean = mean
    #         if mean > max_mean:
    #             max_mean = mean
    #     offset_dist.append(max_mean-min_mean)

    offset_dist = []
    for offset in range(start_offset, end_offset):
        base_dist = []
        for base in test_array[offset]:
            median = np.median(base)
            base_dist.append(median)
        base_dist.sort()
        base_diff = [base_dist[n]-base_dist[n-1] for n in range(1, len(base_dist))]
        total_diff = 0
        for diff in base_diff:
            total_diff += diff
        offset_dist.append(total_diff)

    return offset_dist
def calculate_base_shift(y, fasta_seq, moves, kmer_length):
    best_base_shift = 0
    best_kmer_length = 0
    best_max_dist = 0
    sig_move_offset = 0
    y, fasta_seq, moves = clean_signal(y, fasta_seq, moves)

    test_array = calculate_offset_values(moves, fasta_seq, y, kmer_length, sig_move_offset)
    offset_dist = calculate_offset_distance(kmer_length, test_array)

    start_offset = 0
    end_offset = kmer_length
    max_dist = -1
    max_dist_idx = 0
    for offset in range(start_offset, end_offset):
        # print(offset)
        # print(offset_dist[offset])
        if offset_dist[offset] > max_dist:
            max_dist_idx = offset
            max_dist = offset_dist[offset]
    if best_max_dist < max_dist:
        best_max_dist = max_dist
        best_base_shift = -1*max_dist_idx + sig_move_offset
        best_kmer_length = kmer_length
    # print("best_max_dist: {}".format(best_max_dist))
    # print("best_kmer_length: {}".format(best_kmer_length))
    return best_base_shift


#!/usr/bin/env python
import argparse
import copy
import warnings

import pandas as pd
from plotnine.exceptions import PlotnineWarning

from nanoCEM.cem_utils import read_fasta_to_dic,identify_file_path,build_out_path,extract_kmer_feature,calculate_MANOVA_result
from nanoCEM.plot import current_plot,plot_PCA,MANOVA_plot

warnings.filterwarnings("ignore", category=PlotnineWarning)

def init_parser():
    def add_public_argument(parser_input):
        parser_input.add_argument("--chrom", required=True, help="Gene or chromosome name(head of your fasta file)")
        parser_input.add_argument("--pos", required=True, type=int, help="site of your interest")
        parser_input.add_argument("--len", default=10, type=int, help="region around the position")
        parser_input.add_argument("--strand", default="+", help="Strand of your interest")
        parser_input.add_argument('-r', "--ref", required=True, help="fasta file")
        parser_input.add_argument('-t', "--cpu", default=4, type=int, help="num of process")
        parser_input.add_argument('--norm', action='store_true', help='Turn on the normalization mode')
        parser_input.add_argument('-s', "--subsample_ratio", default=1, type=float,
                                  help="Subsample ratio to select reads")
        parser_input.add_argument('--rna', action='store_true', help='Turn on the RNA mode')
        parser_input.add_argument('--kmer_size', choices=[3,5,7],default=3, help="kmer size for PCA and MANOVA analysis")
        parser_input.add_argument('-o', "--output", default="nanoCEM_result", help="output_file")

    # Define the argument parser
    parser = argparse.ArgumentParser(
        description='A sample tool designed to visualize the features that distinguish between two groups of ONT data at the site level. It supports two re-squiggle pipeline(Tombo and f5c).')
    subparsers = parser.add_subparsers(dest='function')

    # tombo subparser
    parser_tombo = subparsers.add_parser('tombo', help='tackle tombo re-squiggle')
    parser_tombo.add_argument('--basecall_group', default="RawGenomeCorrected_000",
                              help='The attribute group to extract the training data from. e.g. RawGenomeCorrected_000')
    parser_tombo.add_argument('--basecall_subgroup', default='BaseCalled_template',
                              help='Basecall subgroup Nanoraw resquiggle into. Default is BaseCalled_template')
    parser_tombo.add_argument('-i', "--input_fast5", required=True,
                              help="input_fast5_file")
    parser_tombo.add_argument('-c', "--control_fast5",
                              help="control_fast5_file")
    # parser_tombo.add_argument('-b', "--bam", help="bam file to help index to speed up")


    add_public_argument(parser_tombo)

    # f5c subparser
    parser_f5c = subparsers.add_parser('f5c_ev', help='tackle f5c eventalign')
    parser_f5c.add_argument("-i", "--input", required=True,
                            help="blow5_path")
    parser_f5c.add_argument('-c', "--control",
                            help="control_blow5_path")
    parser_f5c.add_argument('--base_shift', action='store_true', help='Turn on the base shift')
    parser_f5c.add_argument('--pore', choices=['r9', 'r10','rna004'], help='Select pore type', default='r9')
    add_public_argument(parser_f5c)

    parser_f5c = subparsers.add_parser('f5c_re', help='tackle f5c re-squiggle')
    parser_f5c.add_argument("-i", "--input", required=True,
                            help="blow5_path")
    parser_f5c.add_argument('-c', "--control",
                            help="control_blow5_path")
    parser_f5c.add_argument('--base_shift',choices=['auto',0,-1,-2,-3,-4,-5,-6,-7,-8], default='auto', help='base shift option')
    parser_f5c.add_argument('--pore', choices=['r9', 'r10','rna004'], help='Select pore type', default='r9')
    add_public_argument(parser_f5c)

    parser_f5c = subparsers.add_parser('move_table', help='tackle move_table from basecaller')
    parser_f5c.add_argument("-i", "--input", required=True,
                            help="blow5_path")
    parser_f5c.add_argument('-c', "--control",
                            help="control_blow5_path")
    parser_f5c.add_argument('-m','--sig_move_offset', type=int, required=True, help='sig_move_offset for squigualiser reform')
    parser_f5c.add_argument('-k','--kmer_length', type=int, required=True, help='kmer_length for squigualiser reform')
    add_public_argument(parser_f5c)
    # parser.set_defaults(function='tombo', chrom="NR_103073.1", pos=2524, len=10, strand='+', cpu=8,norm=True,
    #                     input_fast5='/hdd_data/download/23s_rRNA_A2030/wt/single/', \
    #                     control_fast5='/hdd_data/download/23s_rRNA_A2030/ivt/single/', output='tombo_result_rna',
    #                     subsample_ratio=1, \
    #                     ref="../example/data/23S_rRNA.fasta", \
    #                     basecall_group="RawGenomeCorrected_000", basecall_subgroup="BaseCalled_template", rna=True)
    # parser.set_defaults(function='tombo', chrom="NR_103073.1", pos=2030, len=10, strand='+', cpu=8,norm=True,kmer_size=5,
    #                         input_fast5 = '../example/data/reverse/wt/single/', \
    #                         control_fast5='../example/data/reverse/ivt/single/', output='tombo_result_rna_2030',
    #                         subsample_ratio=1, \
    #                         ref="../example/data/23S_rRNA.fasta", \
    #                          basecall_group="RawGenomeCorrected_000", basecall_subgroup="BaseCalled_template", rna=True)
    # parser.set_defaults(function='tombo', chrom="17", pos=43281378, len=10, strand='+', cpu=8,norm=True,
    #                     input_fast5='../example/dna/hg38_chr17_43281378/WGS_r941/single/', \
    #                     control_fast5='../example/dna/hg38_chr17_43281378/scWGA_941/single/', output='tombo_result_dna',
    #                     subsample_ratio=1, \
    #                     ref="../example/dna/hg38_17.fa", \
    #                     basecall_group="RawGenomeCorrected_000", basecall_subgroup="BaseCalled_template", rna=False)

    # parser.set_defaults(function='move_table', chrom="NR_103073.1", pos=2251, len=5, strand='+', cpu=8,norm=True,
    #                     input='../example/data/wt/basecall',sig_move_offset=0,kmer_length=1,
    #                     control='../example/data/ivt/basecall', output='move_result_rna_2251',
    #                     subsample_ratio=1,
    #                     ref="../example/data/23S_rRNA.fasta", rna=True)
    # parser.set_defaults(function='move_table', chrom="17", pos=43281378, len=10, strand='+', cpu=8,norm=True,
    #                                         input='../example/dna/hg38_chr17_43281378/WGS_r941/basecall',
    #                                         control='../example/dna/hg38_chr17_43281378/scWGA_941/basecall', output='move_result_dna',
    #                     subsample_ratio=1,sig_move_offset=4,kmer_length=5,
    #                     ref="../example/dna/hg38_17.fa", rna=False)
    # parser.set_defaults(function='f5c_re', chrom="17", pos=43281378, len=10, strand='+', cpu=8,base_shift='auto',norm=True,
    #                     input='../example/dna/hg38_chr17_43281378/WGS_r941/file',kmer_size=3,
    #                     control='../example/dna/hg38_chr17_43281378/scWGA_941/file', output='f5c_result_dna_re',
    #                     subsample_ratio=1,rna=False,
    #                     ref="../example/dna/hg38_17.fa", pore='r9')
    # parser.set_defaults(function='f5c_re', chrom="17", pos=39976063, len=10, strand='-', cpu=8, base_shift='auto',
    #                     norm=True,
    #                     input='../example/dna/hg38_chr17_43281378/WGS_r941/file', kmer_size=3,
    #                     control='../example/dna/hg38_chr17_43281378/scWGA_941/file', output='f5c_result_dna_re_re',
    #                     subsample_ratio=1, rna=False,
    #                     ref="../example/dna/hg38_17_rev.fa", pore='r9')

    # parser.set_defaults(function='f5c_re', chrom="NR_103073.1", pos=2524, len=10, strand='+', cpu=8,base_shift=True,norm=True,
    #                     input='/hdd_data/download/23s_rRNA_A2030/wt/file',
    #                     control='/hdd_data/download/23s_rRNA_A2030/ivt/file', output='f5c_result_rna_re_2524',
    #                     subsample_ratio=1,rna=True,
    #                     ref="../example/data/23S_rRNA.fasta", pore='r9')
    # parser.set_defaults(function='f5c_re', chrom="NR_103073.1", pos=875, len=10, strand='-', cpu=8,norm=True,base_shift='auto',pore='r9',
    #                     input='../example/data/wt/file', control='../example/data/ivt/file', output='f5c_result_rna_re_re',
    #                     subsample_ratio=1,kmer_size=3,
    #                     ref="../example/data/reverse/23S_rRNA_re.fasta", rna=True)
    parser.set_defaults(function='f5c_ev', chrom="NR_103073.1", pos=2524, len=5, strand='+', cpu=8,norm=True,base_shift='auto',pore='r9',
                        input='../example/data/wt/file', control='../example/data/ivt/file', output='f5c_result_rna_ev_2524',kmer_size=5,
                        subsample_ratio=1,
                        ref="../example/data/23S_rRNA.fasta", rna=True)
    return parser

if __name__ == '__main__':
    # Parse the arguments
    parser = init_parser()
    args = parser.parse_args()
    # args check
    if args.function is None:
        raise Exception("Please choose function from f5c or tombo, or use -h to view the help document")
    identify_file_path(args.ref)
    fasta = read_fasta_to_dic(args.ref)
    args.pos = args.pos - 1
    length_gene = len(fasta[args.chrom])
    if args.pos + args.len + 1 >= length_gene or args.pos - args.len <= 0:
        raise Exception("The position requested is too close to the border (pos-len>0 and pos+len<length of fasta)")

    # build title
    base_list = fasta[args.chrom][args.pos - args.len:args.pos + args.len + 1].upper()
    title = args.chrom + ':' + str(args.pos - args.len + 1) + '-' + str(args.pos + args.len + 1) + ':' + args.strand
    if args.rna or args.pore == 'rna004':
        args.rna =True
        base_list = base_list.replace("T", "U")
        print("Running RNA mode ...")
        title = 'RNA  ' + title
    else:
        print("Running DNA mode ...")
        title = 'DNA  ' + title

    results_path = args.output
    build_out_path(results_path)
    subsample_ratio = args.subsample_ratio
    single_mode = False

    if args.function == 'tombo':
        from nanoCEM.read_tombo_resquiggle import create_read_list_file, extract_group

        wt_file = create_read_list_file(args.input_fast5, results_path)
        df_wt, aligned_num_wt = extract_group(args, wt_file, subsample_ratio)
        df_wt['Group'] = 'Sample'
        try:
            ivt_file = create_read_list_file(args.control_fast5, results_path)
            df_ivt, aligned_num_ivt = extract_group(args, ivt_file, subsample_ratio)
            df_ivt['Group'] = 'Control'
        except:
            single_mode = True
        if args.control_fast5 is None:
            single_mode = True

    elif args.function == 'f5c_ev':
        from nanoCEM.read_f5c_eventalign import read_blow5
        df_wt, aligned_num_wt, nucleotide_type = read_blow5(args.input, args.pos,args.ref, args.len, args.chrom, args.strand, args.pore,
                                                            subsample_ratio, args.base_shift, args.norm, str(args.cpu),args.rna)
        df_wt['Group'] = 'Sample'
        if nucleotide_type == 'RNA' and not args.rna:
            raise RuntimeError("You need to add --rna to turn on the rna mode")
        try:
            df_ivt, aligned_num_ivt, _ = read_blow5(args.control, args.pos,args.ref, args.len, args.chrom, args.strand, args.pore,
                                                            subsample_ratio, args.base_shift, args.norm, str(args.cpu),args.rna)
            df_ivt['Group'] = 'Control'
            df = pd.concat([df_wt, df_ivt])
        except:
            single_mode =True
        if args.control is None:
            single_mode =True
    elif args.function == 'move_table':
        from nanoCEM.read_move_table import read_basecall_bam
        df_wt, aligned_num_wt = read_basecall_bam(args.input, args.pos, args.ref, args.len, args.chrom,
                                                            args.strand, args.sig_move_offset, args.kmer_length,
                                                            subsample_ratio, args.norm, str(args.cpu),
                                                            args.rna)
        df_wt['Group'] = 'Sample'
        try:
            df_ivt, aligned_num_ivt = read_basecall_bam(args.control, args.pos, args.ref, args.len, args.chrom, args.strand, args.sig_move_offset, args.kmer_length,
                                                    subsample_ratio, args.norm, str(args.cpu),
                                                    args.rna)
            df_ivt['Group'] = 'Control'


        except Exception as e:
            print(e)
            single_mode = True
        if args.control is None:
            single_mode = True
    elif args.function == 'f5c_re':
        from nanoCEM.read_f5c_resquiggle import read_blow5

        df_wt, aligned_num_wt, nucleotide_type = read_blow5(args.input, args.pos,args.ref, args.len, args.chrom, args.strand, args.pore,
                                                            subsample_ratio, args.base_shift, args.norm, str(args.cpu),args.rna)
        df_wt['Group'] = 'Sample'
        if nucleotide_type == 'RNA' and not args.rna:
            raise RuntimeError("You need to add --rna to turn on the rna mode")
        try:
            df_ivt, aligned_num_ivt, _ = read_blow5(args.control, args.pos,args.ref, args.len, args.chrom, args.strand, args.pore,
                                                            subsample_ratio, args.base_shift, args.norm, str(args.cpu),args.rna)
            df_ivt['Group'] = 'Control'

            df = pd.concat([df_wt, df_ivt])
        except:
            single_mode =True
        if args.control is None:
            single_mode =True
    else:
        raise  RuntimeError("Wrong option, select from tombo, move_table, f5c_ev, f5c_re")
    if single_mode:
        df = df_wt
        df_wt['Group'] = 'Single'
        title = title + '   Sample:' + str(aligned_num_wt)
    else:
        df = pd.concat([df_wt, df_ivt])
        title = title + '   Sample:' + str(aligned_num_wt) + '  Control:' + str(aligned_num_ivt)
        category = pd.api.types.CategoricalDtype(categories=['Sample', "Control"], ordered=True)
        df['Group'] = df['Group'].astype(category)

    # save result table
    df_copy = copy.deepcopy(df)
    df_copy['Position'] = df_copy['Position'].astype(int) + 1
    df_copy.to_csv(results_path + '/current_feature.csv', index=False)
    print("Feature file saved  in " + results_path + '/current_feature.csv')
    # draw current feature
    current_plot(df, results_path, args.pos, base_list, title,filter=True)
    if not single_mode:
        if args.kmer_size is None:
            args.kmer_size = 3
        # draw PCA
        kmer_size = args.kmer_size
        # df_copy.loc[:, 'Position'] = df_copy['Position'].astype(int)
        feature_matrix, label = extract_kmer_feature(df_copy,kmer_size, args.pos + 1)
        plot_PCA(feature_matrix, label, results_path)
        # draw p_value
        new_df = calculate_MANOVA_result( args.pos+1,df_copy,args.len,500,args.len,kmer_size)
        MANOVA_plot(new_df, results_path,3)
    print('Finished')




import plotnine as p9
from matplotlib import pyplot as plt
import pandas as pd
from sklearn.model_selection import train_test_split
import xgboost
from nanoCEM.cem_utils import  extract_kmer_feature
from sklearn.metrics import roc_curve, auc,precision_recall_curve
import numpy as np
plt.rcParams['pdf.fonttype'] = 42
results_path='f5c_result_rna_re'
df = pd.read_csv(results_path+'/current_feature.csv')

kmer_list = [3, 5, 7]
final_df=pd.DataFrame()

for item in kmer_list:
    feature_matrix, label = extract_kmer_feature(df, item, 2030)

    label[0] = label[0].apply(lambda x: 1 if x == 'Sample' else 0)

    X = feature_matrix.values
    y = label.values
    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state=2)
    model = xgboost.XGBClassifier()

    # Train the model on the training data
    model.fit(X_train, y_train)
    y_pred = model.predict_proba(X_test)[:, 1]

    # y_pred = model.predict_proba(X)[:,1]
    precision, recall, thresholds = precision_recall_curve(y_test, y_pred)
    auc_value = auc(recall, precision)

    # 创建DataFrame
    df_pr = pd.DataFrame({'Precision': precision[:-1], 'Recall': recall[:-1]})
    df_pr['kmer_length'] = str(item)+'-mer: '+ str(np.round(auc_value,2))
    final_df = pd.concat([final_df,df_pr],axis=0)

plot = p9.ggplot(final_df, p9.aes(x='Recall', y='Precision',color='kmer_length')) \
       + p9.scale_color_manual(['#9d906d','#00c9aa','#9f89ac'])\
       + p9.geom_smooth(method='loess',alpha=0.3) \
       + p9.labs(title='PR Curve') \
       + p9.theme_bw() \
       + p9.theme(
    figure_size=(4, 4),
    panel_grid_minor=p9.element_blank(),
    axis_text=p9.element_text(size=13),
    axis_title=p9.element_text(size=13),
    title=p9.element_text(size=13,hjust=0.5),
    legend_position='bottom',
    legend_title=p9.element_blank(),
    strip_text=p9.element_text(size=13),
    strip_background=p9.element_rect(alpha=0),
)
plot.save(filename=results_path + "/pr_curve.pdf", dpi=300)